@@ -15,7 +15,7 @@ extends 'SQL::Engine::Builder';
 
 use SQL::Validator;
 
-our $VERSION = '0.01'; # VERSION
+our $VERSION = '0.02'; # VERSION
 
 # ATTRIBUTES
 
@@ -49,6 +49,12 @@ has default => (
   opt => 1
 );
 
+has returning => (
+  is => 'ro',
+  isa => 'ArrayRef[HashRef]',
+  opt => 1
+);
+
 # METHODS
 
 method data() {
@@ -74,6 +80,10 @@ method data() {
     $schema->{"default"} = $self->default;
   }
 
+  if ($self->returning) {
+    $schema->{"returning"} = $self->returning;
+  }
+
   return {
     insert => $schema
   }
