@@ -15,7 +15,7 @@ extends 'SQL::Engine::Builder';
 
 use SQL::Validator;
 
-our $VERSION = '0.01'; # VERSION
+our $VERSION = '0.02'; # VERSION
 
 # ATTRIBUTES
 
@@ -43,6 +43,12 @@ has columns => (
   opt => 1
 );
 
+has constraints => (
+  is => 'ro',
+  isa => 'ArrayRef[HashRef]',
+  opt => 1
+);
+
 has query => (
   is => 'ro',
   isa => 'HashRef',
@@ -70,6 +76,10 @@ method data() {
     $schema->{"columns"} = $self->columns;
   }
 
+  if ($self->constraints) {
+    $schema->{"constraints"} = $self->constraints;
+  }
+
   if ($self->query) {
     $schema->{"query"} = $self->query;
   }
