@@ -15,7 +15,7 @@ extends 'SQL::Engine::Builder';
 
 use SQL::Validator;
 
-our $VERSION = '0.01'; # VERSION
+our $VERSION = '0.02'; # VERSION
 
 # ATTRIBUTES
 
@@ -37,6 +37,12 @@ has where => (
   opt => 1
 );
 
+has returning => (
+  is => 'ro',
+  isa => 'ArrayRef[HashRef]',
+  opt => 1
+);
+
 # METHODS
 
 method data() {
@@ -54,6 +60,10 @@ method data() {
     $schema->{"where"} = $self->where;
   }
 
+  if ($self->returning) {
+    $schema->{"returning"} = $self->returning;
+  }
+
   return {
     update => $schema
   }
