@@ -17,7 +17,7 @@ use SQL::Validator;
 
 use Scalar::Util ();
 
-our $VERSION = '0.01'; # VERSION
+our $VERSION = '0.02'; # VERSION
 
 # ATTRIBUTES
 
@@ -61,8 +61,27 @@ method binding(Str $name) {
 }
 
 method column_change(HashRef $data) {
+  my $sql = [];
 
-  return $self;
+  # alter table
+  push @$sql, $self->term(qw(alter table));
+
+  # safe
+  push @$sql, $self->term(qw(if exists)) if $data->{safe};
+
+  # for
+  push @$sql, $self->table($data->{for});
+
+  # column
+  push @$sql, $self->term(qw(alter column));
+
+  # column specification
+  push @$sql, $self->column_specification($data->{column});
+
+  # sql statement
+  my $result = join ' ', @$sql;
+
+  return $self->operation($result);
 }
 
 method column_create(HashRef $data) {
@@ -331,6 +350,13 @@ method criterion(HashRef $data) {
       @$cond[1 .. $#$cond];
   }
 
+  if (my $cond = $data->{"is"}) {
+    return sprintf '(%s)',
+      (ref($cond) eq 'HASH')
+        ? $self->expression($cond)
+        : join(sprintf(' %s ', $self->term('and')), @{$self->criteria($cond)});
+  }
+
   if (my $cond = $data->{"is-null"}) {
     return sprintf '%s IS NULL', $self->expression($cond);
   }
@@ -352,6 +378,13 @@ method criterion(HashRef $data) {
     return sprintf '%s != %s', map $self->expression($_), @$cond;
   }
 
+  if (my $cond = $data->{"not"}) {
+    return sprintf 'NOT (%s)',
+      (ref($cond) eq 'HASH')
+        ? $self->expression($cond)
+        : join(sprintf(' %s ', $self->term('and')), @{$self->criteria($cond)});
+  }
+
   if (my $cond = $data->{"not-null"}) {
     return sprintf '%s IS NOT NULL', $self->expression($cond);
   }
@@ -382,6 +415,12 @@ method delete(HashRef $data) {
       join(sprintf(' %s ', $self->term('and')), @{$self->criteria($where)});
   }
 
+  # returning (postgres)
+  if (my $returning = $data->{returning}) {
+    push @$sql, $self->term('returning'),
+      sprintf('(%s)', join(', ', map $self->expression($_), @$returning));
+  }
+
   # sql statement
   my $result = join ' ', @$sql;
 
@@ -536,6 +575,24 @@ method expression(Any $data) {
   }
 }
 
+method join_option(Maybe[Str] $name) {
+  if (lc($name) eq "left-join") {
+    return $self->term(qw(left join));
+  }
+  elsif (lc($name) eq "right-join") {
+    return $self->term(qw(right join));
+  }
+  elsif (lc($name) eq "full-join") {
+    return $self->term(qw(full join));
+  }
+  elsif (lc($name) eq "inner-join") {
+    return $self->term(qw(inner join));
+  }
+  else {
+    return $self->term(qw(join));
+  }
+}
+
 method name(Any @args) {
 
   return join '.', map { /\W/ ? $_ : $self->wrap($_) } grep {defined} @args;
@@ -790,6 +847,12 @@ method insert(HashRef $data) {
     push @$sql, $self->term('default'), $self->term('values');
   }
 
+  # returning (postgres)
+  if (my $returning = $data->{returning}) {
+    push @$sql, $self->term('returning'),
+      sprintf('(%s)', join(', ', map $self->expression($_), @$returning));
+  }
+
   # sql statement
   my $result = join ' ', @$sql;
 
@@ -850,6 +913,11 @@ method select(HashRef $data) {
     push @$sql, join(', ', map $self->expression($_), @$columns);
   }
 
+  # into (mssql)
+  if (my $into = $data->{into}) {
+    push @$sql, $self->term('into'), $self->name($into);
+  }
+
   # from
   push @$sql, $self->term('from'),
     ref($data->{from}) eq 'ARRAY'
@@ -859,7 +927,7 @@ method select(HashRef $data) {
   # joins
   if (my $joins = $data->{joins}) {
     for my $join (@$joins) {
-      push @$sql, $self->term('join'), $self->table($join->{with});
+      push @$sql, $self->join_option($join->{type}), $self->table($join->{with});
       push @$sql, $self->term('on'),
         join(
         sprintf(' %s ', $self->term('and')),
@@ -956,10 +1024,71 @@ method table_create(HashRef $data) {
     ($data->{safe} ? $self->term(qw(if not exists)) : ()),
     $self->name($data->{name});
 
+  # body
+  my $body = [];
+
   # columns
   if (my $columns = $data->{columns}) {
-    push @$sql, sprintf('(%s)', join ', ',
-      map $self->column_specification($_), @$columns);
+    push @$body, map $self->column_specification($_), @$columns;
+  }
+
+  # constraints
+  if (my $constraints = $data->{constraints}) {
+    # unique
+    for my $constraint (grep {$_->{unique}} @{$constraints}) {
+      if (my $unique = $constraint->{unique}) {
+        my $name = $self->index_name({
+          for => $data->{for},
+          name => $unique->{name},
+          columns => [map +{column => $_}, @{$unique->{columns}}],
+          unique => 1,
+        });
+        push @$body, join ' ', $self->term('constraint'), $name,
+          $self->term('unique'), sprintf '(%s)', join ', ',
+          map $self->name($_), @{$unique->{columns}};
+      }
+    }
+    # foreign
+    for my $constraint (grep {$_->{foreign}} @{$constraints}) {
+      if (my $foreign = $constraint->{foreign}) {
+        my $name = $self->constraint_name({
+          source => {
+            table => $data->{name},
+            column => $foreign->{column}
+          },
+          target => $foreign->{reference},
+          name => $foreign->{name}
+        });
+        push @$body, join ' ', $self->term('constraint'), $name,
+          $self->term(qw(foreign key)),
+          sprintf('(%s)', $self->name($foreign->{column})),
+          $self->term(qw(references)),
+          sprintf('%s (%s)',
+          $self->table($foreign->{reference}),
+          $self->name($foreign->{reference}{column})),
+          (
+          $foreign->{on}{delete}
+          ? (
+            $self->term(qw(on delete)),
+            $self->constraint_option($foreign->{on}{delete})
+            )
+          : ()
+          ),
+          (
+          $foreign->{on}{update}
+          ? (
+            $self->term(qw(on update)),
+            $self->constraint_option($foreign->{on}{update})
+            )
+          : ()
+          );
+      }
+    }
+  }
+
+  # definition
+  if (@$body) {
+    push @$sql, sprintf('(%s)', join ', ', @$body);
   }
 
   # query
@@ -1023,6 +1152,7 @@ method term(Str @args) {
 }
 
 method transaction(HashRef $data) {
+  $DB::single=1;
   $self->operation($self->term('begin', 'transaction'));
   $self->process($_) for @{$data->{queries}};
   $self->operation($self->term('commit'));
@@ -1341,6 +1471,12 @@ method update(HashRef $data) {
       join(sprintf(' %s ', $self->term('and')), @{$self->criteria($where)});
   }
 
+  # returning (postgres)
+  if (my $returning = $data->{returning}) {
+    push @$sql, $self->term('returning'),
+      sprintf('(%s)', join(', ', map $self->expression($_), @$returning));
+  }
+
   # sql statement
   my $result = join ' ', @$sql;
 
@@ -2349,7 +2485,7 @@ transaction.
     }
   });
 
-  $grammar->column_change($grammar->schema->{'transaction'});
+  $grammar->transaction($grammar->schema->{'transaction'});
 
 =back
 
